"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.readUser = exports.readDepositor = exports.readSite = exports.getIsByUserAllowedDepositorChange = exports.getIsByUserAllowedSiteChange = exports.getByUserCurrentDepositorCode = exports.getByUserCurrentSiteCode = exports.getDefaultSite = void 0;
const xtrem_decimal_1 = require("@sage/xtrem-decimal"), typesLib = xtrem_decimal_1;
const user_parameters_values_1 = require("@sage/wh-system/lib/client-functions/user-parameters-values");
const ui = require("@sage/xtrem-ui");
/**
 * get default site for the user for GEODE module
 * @param user
 * @returns default site for the user
 */
function getDefaultSite(user) {
    return (0, user_parameters_values_1.getUserProfile)(user) ? (0, user_parameters_values_1.getDefaultSiteFromModule)(user, 2) : null;
}
exports.getDefaultSite = getDefaultSite;
/**
 * get string user code
 * @param user
 * @returns
 */
function _getUserCode(user) {
    return typesLib.strictEq(typeof user, 'string') ? user?.toUpperCase() : user?.code;
}
/**
 * get site for the user for GEODE module
 * @param user
 * @returns default site for the user
 */
async function getByUserCurrentSiteCode(pageInstance, userCode) {
    return (await _readByUserParameterValueString(pageInstance, 'FCY', _getUserCode(userCode))) ?? null;
}
exports.getByUserCurrentSiteCode = getByUserCurrentSiteCode;
/**
 * get depositor for the user for GEODE module
 * @param userCode
 * @returns default site for the user
 */
async function getByUserCurrentDepositorCode(pageInstance, userCode) {
    return (await _readByUserParameterValueString(pageInstance, 'FCY', _getUserCode(userCode)))
        ? ((await _readByUserParameterValueString(pageInstance, 'DEP', _getUserCode(userCode))) ?? null)
        : null;
}
exports.getByUserCurrentDepositorCode = getByUserCurrentDepositorCode;
/**
 * Check if you can enter the site
 * @param pageInstance current page instance
 * @param userCode
 * @returns
 */
async function getIsByUserAllowedSiteChange(pageInstance, userCode) {
    return !!(await _readByUserParameterValueBoolean(pageInstance, 'MULFCY', _getUserCode(userCode)));
}
exports.getIsByUserAllowedSiteChange = getIsByUserAllowedSiteChange;
/**
 * @param pageInstance current page instance
 * Check if you can enter the depositor
 * @param userCode
 * @returns
 */
async function getIsByUserAllowedDepositorChange(pageInstance, userCode) {
    return !!(await _readByUserParameterValueBoolean(pageInstance, 'MULDEP', _getUserCode(userCode)));
}
exports.getIsByUserAllowedDepositorChange = getIsByUserAllowedDepositorChange;
/**
 * Read record section
 */
/**
 * Read site from given site
 * Only instance with property code
 * @param pageInstance
 * @param siteCode
 * @param depositorCode
 * @returns site when exist
 */
async function readSite(pageInstance, siteCode) {
    try {
        return (await pageInstance.$.graph
            .node('@sage/wh-system/Site')
            .read({
            code: true,
        }, `${siteCode?.toUpperCase()}`)
            .execute());
    }
    catch (error) {
        return null;
    }
}
exports.readSite = readSite;
/**
 * Read depositor from given site and depositor codes
 * Only instance with property site and code
 * @param pageInstance
 * @param siteCode
 * @param depositorCode
 * @returns depositor when exist
 */
async function readDepositor(pageInstance, siteCode, depositorCode) {
    try {
        return (await pageInstance.$.graph
            .node('@sage/wh-master-data/Depositor')
            .read({
            site: {
                code: true,
            },
            code: true,
        }, `${siteCode?.toUpperCase()}|${depositorCode?.toUpperCase()}`)
            .execute());
    }
    catch (error) {
        return null;
    }
}
exports.readDepositor = readDepositor;
/**
 *  Read user from given user code for print destinations and site depositor
 * @param pageInstance
 * @param userCode
 * @returns
 */
async function readUser(pageInstance, userCode) {
    try {
        return (await pageInstance.$.graph
            .node('@sage/wh-system/User')
            .read({
            code: true,
            name: true,
            functionProfile: {
                profile: true,
                userProfileDefaultSites: {
                    query: {
                        edges: {
                            node: {
                                site: {
                                    code: true,
                                },
                            },
                        },
                    },
                },
            },
            profession: {
                profession: true,
                functionProfile: {
                    profile: true,
                    userProfileDefaultSites: {
                        query: {
                            edges: {
                                node: {
                                    site: {
                                        code: true,
                                    },
                                },
                            },
                        },
                    },
                },
            },
            userDestinations: {
                query: {
                    edges: {
                        node: {
                            code: true,
                            destination: {
                                code: true,
                                description: true,
                                destination: true,
                            },
                        },
                    },
                },
            },
            currentDepositorCode: true,
            currentSiteCode: true,
            defaultSiteCode: true,
            isAllowedDepositorChange: true,
            isAllowedSiteChange: true,
        }, `${userCode?.toUpperCase() ?? pageInstance.$.userCode?.toUpperCase()}`)
            .execute());
    }
    catch (error) {
        return null;
    }
}
exports.readUser = readUser;
/**
 * General section for current user
 */
/**
 * Read the value of a string parameter
 * @param pageInstance
 * @param parameter
 * @returns
 */
async function _readParameterValueString(pageInstance, parameter) {
    try {
        return (await pageInstance.$.graph
            .node('@sage/wh-system/UserParameterValue')
            .mutations.readParameterValueString({ value: true }, {
            parameters: {
                code: parameter,
            },
        })
            .execute())?.value;
    }
    catch (e) {
        ui.console.error(`Error reading parameter value ${parameter} : `, e.message);
        return undefined;
    }
}
/**
 *  Read the value of an integer parameter
 * @param pageInstance
 * @param parameter
 * @returns
 */
async function _readParameterValueInteger(pageInstance, parameter) {
    try {
        return (await pageInstance.$.graph
            .node('@sage/wh-system/UserParameterValue')
            .mutations.readParameterValueInteger({ value: true }, {
            parameters: {
                code: parameter,
            },
        })
            .execute())?.value;
    }
    catch (e) {
        ui.console.error(`Error reading parameter value ${parameter} : `, e.message);
        return undefined;
    }
}
/**
 *  Read the value of a date parameter
 * @param pageInstance
 * @param parameter
 * @returns
 */
async function _readParameterValueDate(pageInstance, parameter) {
    try {
        return (await pageInstance.$.graph
            .node('@sage/wh-system/UserParameterValue')
            .mutations.readParameterValueDate({ value: true }, {
            parameters: {
                code: parameter,
            },
        })
            .execute())?.value;
    }
    catch (e) {
        ui.console.error(`Error reading parameter value ${parameter} : `, e.message);
        return undefined;
    }
}
/**
 * Read a parameter value as a boolean
 * @param pageInstance
 * @param parameter
 * @returns
 */
async function _readParameterValueBoolean(pageInstance, parameter, user) {
    const _value = await _readParameterValueInteger(pageInstance, parameter);
    return _value ? typesLib.strictEq(_value, 2) : undefined;
}
/**
 * Sticker site depositor section (need administrator rights for using user)
 */
/**
 * Read the value of a string parameter for given user
 * @param pageInstance
 * @param parameter
 * @param user
 * @returns
 */
async function _readByUserParameterValueString(pageInstance, parameter, user) {
    try {
        return (await pageInstance.$.graph
            .node('@sage/wh-system/UserParameterValue')
            .mutations.readByUserParameterValueString({ value: true }, {
            parameters: {
                user: user,
                code: parameter,
            },
        })
            .execute())?.value;
    }
    catch (e) {
        ui.console.error(`Error reading parameter value ${parameter} : `, e.message);
        return undefined;
    }
}
/**
 *  Read the value of an integer parameter for given user
 * @param pageInstance
 * @param parameter
 * @param user
 * @returns
 */
async function _readByUserParameterValueInteger(pageInstance, parameter, user) {
    try {
        return (await pageInstance.$.graph
            .node('@sage/wh-system/UserParameterValue')
            .mutations.readByUserParameterValueInteger({ value: true }, {
            parameters: {
                user: user,
                code: parameter,
            },
        })
            .execute())?.value;
    }
    catch (e) {
        ui.console.error(`Error reading parameter value ${parameter} : `, e.message);
        return undefined;
    }
}
/**
 *  Read the value of a date parameter for given user
 * @param pageInstance
 * @param parameter
 * @param user
 * @returns
 */
async function _readByUserParameterValueDate(pageInstance, parameter, user) {
    try {
        return (await pageInstance.$.graph
            .node('@sage/wh-system/UserParameterValue')
            .mutations.readByUserParameterValueDate({ value: true }, {
            parameters: {
                user: user,
                code: parameter,
            },
        })
            .execute())?.value;
    }
    catch (e) {
        ui.console.error(`Error reading parameter value ${parameter} : `, e.message);
        return undefined;
    }
}
/**
 * Read a parameter value as a boolean for given user
 * @param pageInstance
 * @param parameter
 * @param user
 * @returns
 */
async function _readByUserParameterValueBoolean(pageInstance, parameter, user) {
    const _value = await _readByUserParameterValueInteger(pageInstance, parameter, user);
    return _value ? typesLib.strictEq(_value, 2) : undefined;
}
//# sourceMappingURL=user-extension.js.map