"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.readDepositor = exports.readUser = exports.readSite = exports.getIsUserAllowedDepositorChange = exports.getIsUserAllowedSiteChange = exports.getByUserCurrentDepositorCode = exports.getByUserCurrentSiteCode = exports.getIsAllowedDepositorChange = exports.getIsAllowedSiteChange = exports.getCurrentDepositorCode = exports.getCurrentSiteCode = exports.getDefaultSiteCode = void 0;
const xtrem_decimal_1 = require("@sage/xtrem-decimal"), typesLib = xtrem_decimal_1;
const whSystem = require("@sage/wh-system");
const user_parameters_values_1 = require("@sage/wh-system/build/lib/functions/user-parameters-values");
const xtrem_core_1 = require("@sage/xtrem-core");
const whMasterData = require("..");
/**
 * get default site code
 * @param _this
 * @returns default site for current user
 */
async function getDefaultSiteCode(_this) {
    return (await (0, user_parameters_values_1.getDefaultSiteFromModule)(_this, 2))?.code ?? '';
}
exports.getDefaultSiteCode = getDefaultSiteCode;
/**
 *  get site code for current user
 * @param _this
 * @param user
 * @returns
 */
async function getCurrentSiteCode(_this) {
    return ((await whSystem.nodes.UserParameterValue.readParameterValueString(_this.$.context, {
        code: 'FCY',
    }))?.value ?? null);
}
exports.getCurrentSiteCode = getCurrentSiteCode;
/**
 * get depositor code for current user
 * @param _this
 * @param user
 * @returns
 */
async function getCurrentDepositorCode(_this) {
    return ((await whSystem.nodes.UserParameterValue.readParameterValueString(_this.$.context, {
        code: 'DEP',
    }))?.value ?? null);
}
exports.getCurrentDepositorCode = getCurrentDepositorCode;
/**
 * Read the boolean value of the parameter for current user
 * @param parameterCode
 * @returns
 */
async function _readParameterValueBoolean(_this, parameterCode) {
    return (typesLib.strictEq((await whSystem.nodes.UserParameterValue.readParameterValueInteger(_this.$.context, {
        code: parameterCode,
    }))?.value, 2));
}
/**
 * Check if you can enter the site
 * @param _this
 * @returns
 */
async function getIsAllowedSiteChange(_this) {
    const _result = await _readParameterValueBoolean(_this, 'MULFCY');
    return _result;
}
exports.getIsAllowedSiteChange = getIsAllowedSiteChange;
/**
 * Check if you can enter the depositor
 *  @param _this
 * @returns
 */
async function getIsAllowedDepositorChange(_this) {
    const _result = await _readParameterValueBoolean(_this, 'MULDEP');
    return _result;
}
exports.getIsAllowedDepositorChange = getIsAllowedDepositorChange;
/**
 * get site code for given user
 * @param this
 * @param userCode
 * @returns default site for the user
 */
async function getByUserCurrentSiteCode(_this, userCode) {
    return ((await whSystem.nodes.UserParameterValue.readByUserParameterValueString(_this.$.context, {
        code: 'FCY',
        user: userCode?.toUpperCase(),
    }))?.value ?? '');
}
exports.getByUserCurrentSiteCode = getByUserCurrentSiteCode;
/**
 * get depositor code for given user
 *  @param this
 * @param user
 * @returns default depositor for the user
 */
async function getByUserCurrentDepositorCode(_this, userCode) {
    return (await getByUserCurrentSiteCode(_this, userCode))
        ? ((await whSystem.nodes.UserParameterValue.readByUserParameterValueString(_this.$.context, {
            code: 'DEP',
            user: userCode?.toUpperCase(),
        }))?.value ?? '')
        : '';
}
exports.getByUserCurrentDepositorCode = getByUserCurrentDepositorCode;
/**
 * Read the boolean value of the parameter for given user
 * @param _this
 * @param parameterCode
 * @param userCode
 * @returns
 */
async function _readByUserParameterValueBoolean(_this, parameterCode, userCode) {
    return (typesLib.strictEq((await whSystem.nodes.UserParameterValue.readByUserParameterValueInteger(_this.$.context, {
        code: parameterCode,
        user: userCode?.toUpperCase(),
    }))?.value, 2));
}
/**
 * Check if the given user can enter the site
 * @param _this
 * @param userCode
 * @returns
 */
async function getIsUserAllowedSiteChange(_this, userCode) {
    const _result = await _readByUserParameterValueBoolean(_this, 'MULFCY', userCode);
    return _result;
}
exports.getIsUserAllowedSiteChange = getIsUserAllowedSiteChange;
/**
 * Check if the given user can enter the depositor
 * @param _this
 * @param userCode
 * @returns
 */
async function getIsUserAllowedDepositorChange(_this, userCode) {
    const _result = await _readByUserParameterValueBoolean(_this, 'MULDEP', userCode);
    return _result;
}
exports.getIsUserAllowedDepositorChange = getIsUserAllowedDepositorChange;
/**
 * Read site from given site
 * Only instance with property code
 * @param _this
 * @param siteCode
 * @returns site when exist
 */
async function readSite(_this, siteCode) {
    if (!siteCode) {
        throw new xtrem_core_1.BusinessRuleError('No site code found');
    }
    try {
        return await _this.$.context.tryRead(whSystem.nodes.Site, {
            code: siteCode,
        });
    }
    catch (error) {
        _this.$.context.logger.error(error);
        return null;
    }
}
exports.readSite = readSite;
/**
 *  Read user from given user code for print destinations and site depositor
 * @param pageInstance
 * @param userCode
 * @returns
 */
async function readUser(_this, userCode) {
    const _userCode = userCode?.toUpperCase() ?? (await _this.$.context.user)?.userName?.toUpperCase();
    if (!_userCode) {
        throw new xtrem_core_1.BusinessRuleError('No user code found');
    }
    try {
        return await _this.$.context.tryRead(whSystem.nodes.User, {
            code: _userCode,
        });
    }
    catch (error) {
        _this.$.context.logger.error(error);
        return null;
    }
}
exports.readUser = readUser;
/**
 * Read record section
 */
/**
 * Read depositor from given site and depositor codes
 * Only instance with property site and code
 * @param _this
 * @param siteCode
 * @param depositorCode
 * @returns depositor when exist
 */
async function readDepositor(_this, siteCode, depositorCode) {
    if (!siteCode) {
        throw new xtrem_core_1.BusinessRuleError('No site code found');
    }
    if (!depositorCode) {
        throw new xtrem_core_1.BusinessRuleError('No depositor code found');
    }
    try {
        return await _this.$.context.tryRead(whMasterData.nodes.Depositor, {
            site: siteCode?.toUpperCase(),
            code: depositorCode?.toUpperCase(),
        });
    }
    catch (error) {
        _this.$.context.logger.error(error);
        return null;
    }
}
exports.readDepositor = readDepositor;
//# sourceMappingURL=user-extension.js.map